"""Collection of descriptive statistics."""
from .expected import E
from .conditional import E_cond
from .covariance import Cov
from .kurtosis import Kurt
from .percentile import Perc
from .quantity_of_interest import QoI_Dist
from .skewness import Skew
from .standard_deviation import Std
from .variance import Var

from .correlation import *
from .sensitivity import *
